from django.db import IntegrityError
from django.shortcuts import render, redirect, get_object_or_404
from account.models import Account 
from django.core.paginator import Paginator
from django.http import HttpResponse
import pandas as pd 

from django.contrib.auth.decorators import login_required 
from django.http import HttpResponseForbidden
from django.contrib import messages

from categories.models import Category
from quotations.models import Inquiry
from stock.models import Stock
from suppliers.models import Supplier
from uom.models import UnitOfMeasure
 
from .models import Product

from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden
from django.shortcuts import render
from django.contrib import messages 

import csv
from django.http import HttpResponse
from openpyxl import Workbook

import csv
from django.http import HttpResponse

@login_required
def products(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            # Get filter parameters from the request
            code = request.GET.get('code', '').strip()
            product_name = request.GET.get('product_name', '').strip()
            category = request.GET.get('category', '')
            product_type = request.GET.get('type', '')

            # Start with all products
            products = Product.objects.all()

            # Apply filters if provided
            if code:
                products = products.filter(pid__icontains=code)
            if product_name:
                products = products.filter(product_name__icontains=product_name)
            if category:
                products = products.filter(category_id=category)
            if product_type:
                products = products.filter(productType=product_type)

            # Get categories for the filter dropdown
            categories = Category.objects.all()

            # Check if export request is made
            if 'export' in request.GET and request.GET['export'] == 'true':
                response = HttpResponse(content_type='text/csv')
                response['Content-Disposition'] = 'attachment; filename="products.csv"'

                writer = csv.writer(response)
                # Write the header row
                writer.writerow([
                    'Product ID', 'Product Name', 'Category', 'Type',
                    'Unit of Measure', 'Supplier', 'Price', 'Quantity in Stock'
                ])

                # Write data rows
                for product in products:
                    writer.writerow([
                        product.pid,
                        product.product_name,
                        product.category.name if product.category else "N/A",
                        product.productType,
                        product.uom,
                        product.supplier_id.supplier_name if product.supplier_id else "N/A",
                        product.price,
                        product.quantity_in_stock
                    ])

                return response

            # Render the page with filtered products and filter data
            return render(request, "inventory/product/products.html", {
                'products': products,
                'categories': categories,
                'code': code,
                'product_name': product_name,
                'selected_category': category,
                'type': product_type
            })
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return render(request, "inventory/product/products.html", {
                'products': [],
                'categories': Category.objects.all(),
                'code': '',
                'product_name': '',
                'selected_category': '',
                'type': ''
            })
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

def export_to_excel(products):
    # Create a workbook and worksheet
    wb = Workbook()
    ws = wb.active
    ws.title = "Products"

    # Write header row
    headers = ["CODE", "Product Name", "Category", "Type", "Unit of Measure", "Supplier", "Price", "Quantity in Stock"]
    ws.append(headers)

    # Write product data
    for product in products:
        ws.append([
            product.pid,
            product.product_name,
            product.category.name if product.category else "",
            product.productType,
            product.uom,
            product.supplier_id,
            product.price,
            product.quantity_in_stock,
        ])

    # Prepare the response
    response = HttpResponse(content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
    response["Content-Disposition"] = 'attachment; filename="products.xlsx"'
    wb.save(response)
    return response

    

from decimal import Decimal
from gauge.models import Gauge

@login_required
def add_product(request):
    try:
        categories = Category.objects.all()
        uoms = UnitOfMeasure.objects.all()
        suppliers = Supplier.objects.all()
        gauges = Gauge.objects.all()

        context = {
            'categories': categories,
            'uoms': uoms,
            'suppliers': suppliers,
            'gauges' : gauges
        }

        if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
            if request.method == 'POST':
                product_data = {
                    'product_name': request.POST.get('product_name'),
                    'product_description': request.POST.get('product_description'),
                    'category_id': request.POST.get('category'),
                    'uom_id': request.POST.get('uom'),
                    'supplier_id': request.POST.get('supplier'),
                    'gauge': request.POST.get('gauge'),
                    'finish': request.POST.get('finish'),
                    'manufacturer': request.POST.get('manufacturer'),
                    'active_status': request.POST.get('active_status'),
                    'price': Decimal(request.POST.get('price', 0)),
                    'min_price': Decimal(request.POST.get('min_price', 0)),
                    'discounted_price': Decimal(request.POST.get('discounted_price', 0)),
                    'is_profiling': request.POST.get('is_profiling') == "true",  # Convert to Boolean
                    'profiling_price': Decimal(request.POST.get('profiling_price', 0)) if request.POST.get('is_profiling') == "true" else 0,
                    'sku': request.POST.get('sku'),
                    'barcode': request.POST.get('barcode'),
                    'quantity_in_stock': Decimal(request.POST.get('quantity_in_stock', 0)),
                    'min_stock_level': Decimal(request.POST.get('min_stock_level', 0)),
                    'max_stock_level': Decimal(request.POST.get('max_stock_level', 0)),
                    'reorder_point': Decimal(request.POST.get('reorder_point', 0)),
                    'weight': Decimal(request.POST.get('weight', 0)) if request.POST.get('weight') else None,
                    'dimensions': request.POST.get('dimensions'),
                    'productType': request.POST.get('productType'),
                    'added_by': request.user
                }

                print(product_data)

                if product_data['is_profiling'] and product_data['profiling_price'] <= 0:
                    raise ValueError("Profiling price must be greater than 0 if profiling is enabled.")


                category = Category.objects.get(id=product_data['category_id']) if product_data['category_id'] else None
                uom = UnitOfMeasure.objects.get(id=product_data['uom_id']) if product_data['uom_id'] else None
                supplier = Supplier.objects.get(id=product_data['supplier_id']) if product_data['supplier_id'] else None
                gauge = Gauge.objects.get(gauge_id=product_data['gauge']) if product_data['gauge'] else None

                product_data.update({
                    'category': category,
                    'uom': uom,
                    'supplier_id': supplier,
                    'gauge':gauge
                })

                product = Product.objects.create(**product_data)

                images = request.FILES.getlist('images[]')
                for image in images:
                    product.images.save(image.name, image)

                messages.success(request, 'Product added successfully.')
                return redirect('inventory:products')

            return render(request, "inventory/product/add_product.html", context)
        else:
            return HttpResponseForbidden("You are not authorized to view this page.")
    except IntegrityError as e:
        messages.error(request, f"Data integrity error: {str(e)}")
        return render(request, "inventory/product/add_product.html", context)
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return render(request, "inventory/product/add_product.html", context)


@login_required
def edit_product(request, product_id):
    if not (request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:

        product = get_object_or_404(Product, product_id=product_id)
        categories = Category.objects.all()
        uoms = UnitOfMeasure.objects.all()
        suppliers = Supplier.objects.all()
        gauges = Gauge.objects.all()

        context = {
            'product': product,
            'categories': categories,
            'uoms': uoms,
            'suppliers': suppliers,
            'gauges' : gauges
        }

        if request.method == 'POST':
            product.product_name = request.POST.get('product_name')
            product.product_description = request.POST.get('product_description')
            gauge_id = request.POST.get('gauge')
            if gauge_id:
                product.gauge = get_object_or_404(Gauge, gauge_id=gauge_id)
            category_id = request.POST.get('category')
            uom_id = request.POST.get('uom')
            finish_value = request.POST.get('finish')
            if isinstance(finish_value, tuple):
                finish_value = finish_value[0]  # Extract the first element if it's a tuple

            product.finish = finish_value
            supplier_id = request.POST.get('supplier')
            product.manufacturer = request.POST.get('manufacturer')
            product.active_status = request.POST.get('active_status')
            product.price = Decimal(request.POST.get('price', 0))
            product.min_price = Decimal(request.POST.get('min_price', 0))
            product.discounted_price = Decimal(request.POST.get('discounted_price', 0))
            product.sku = request.POST.get('sku')
            product.barcode = request.POST.get('barcode')
            product.quantity_in_stock = Decimal(request.POST.get('quantity_in_stock', 0))
            product.min_stock_level = Decimal(request.POST.get('min_stock_level', 0))
            product.max_stock_level = Decimal(request.POST.get('max_stock_level', 0))
            product.reorder_point = Decimal(request.POST.get('reorder_point', 0))
            product.weight = Decimal(request.POST.get('weight', 0)) if request.POST.get('weight') else None
            product.dimensions = request.POST.get('dimensions')
            product.productType = request.POST.get('productType')
            product.is_profiling = request.POST.get('is_profiling') == "true"  
            product.profiling_price = Decimal(request.POST.get('profiling_price', 0)) if request.POST.get('profiling_price') else 0


            product.category = Category.objects.get(id=category_id) if category_id else None
            product.uom = UnitOfMeasure.objects.get(id=uom_id) if uom_id else None
            product.supplier_id = Supplier.objects.get(id=supplier_id) if supplier_id else None

            product.save()

            images = request.FILES.getlist('images[]')
            for image in images:
                product.images.save(image.name, image)

            messages.success(request, 'Product updated successfully.')
            return redirect('inventory:products')

        return render(request, "inventory/product/edit_product.html", context)
    except IntegrityError as e:
        messages.error(request, f"Data integrity error: {str(e)}")
        return render(request, "inventory/product/edit_product.html", context)
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return render(request, "inventory/product/edit_product.html", context)
    

@login_required
def delete_product(request, product_id):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            product = get_object_or_404(Product, product_id=product_id)
            product_name = product.product_name
            product.delete()
            messages.success(request, f"{product_name} has been deleted successfully.")
        except Exception as e:
            messages.error(request, f"An error occurred while deleting the product: {str(e)}")
        return redirect("inventory:products")
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")
    


from django.http import JsonResponse
from .models import Product

from django.http import JsonResponse
from django.contrib import messages
from .models import Product
from django.db.models import Q

def get_products(request):
    try:
        # Get the optional search term from query parameters
        search_term = request.GET.get('search', '').strip()  # Default to empty string if not provided

        # Filter products by productType and optionally by search term
        products = Product.objects.filter(productType='Trading Item')

        if search_term:
            # Filter products based on search term in the code or name
            products = products.filter(
                Q(product_name__icontains=search_term) | Q(pid__icontains=search_term)
            )

        # Serialize the filtered products
        serialized_products = [
            {
                "code": product.pid,
                "name": product.product_name,
                "quantity": product.quantity_in_stock,
                "price": str(product.price),
                "min_price":str(product.min_price),
                "image_url": product.images.url if product.images else ''
            }
            for product in products
        ]
        return JsonResponse(serialized_products, safe=False)
    
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return JsonResponse([], safe=False)


from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status
from .models import Product
from .serializers import ProductSerializer
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

# @api_view(['GET'])
# def get_product_details_api(request, product_id):
#     try:
#         product = Product.objects.get(pk=product_id)
        
#         # Get stock adjustments for the product
#         stock_adjustments = Stock.objects.filter(product=product).order_by('-created_on')
        
#         # Get inquiries related to the product
#         inquiries = Inquiry.objects.filter(product=product).order_by('-created_at')

#         # Calculate total amount made from inquiries
#         total_amount_from_inquiries = inquiries.aggregate(total_amount=Sum('amount'))['total_amount']

#         # Query sales data for the current year before pagination
#         current_year = datetime.datetime.now().year
#         monthly_sales = inquiries.filter(created_at__year=current_year).annotate(
#             month=ExtractMonth('created_at')).values('month').annotate(
#             total_sales=Count('id')).order_by('month')

#         # Prepare data for monthly sales chart
#         sales_data = {
#             'labels': [datetime.date(2000, month, 1).strftime('%B') for month in range(1, 13)],  # Month names
#             'data': [monthly_sales.get(month=month)['total_sales'] if monthly_sales.filter(month=month).exists() else 0 for month in range(1, 13)]  # Sales counts
#         }

#         # Paginate stock adjustments
#         stock_page = request.GET.get('stock_page', 1)
#         stock_paginator = Paginator(stock_adjustments, 10)  # Show 10 adjustments per page
#         try:
#             stock_adjustments = stock_paginator.page(stock_page)
#         except PageNotAnInteger:
#             stock_adjustments = stock_paginator.page(1)
#         except EmptyPage:
#             stock_adjustments = stock_paginator.page(stock_paginator.num_pages)

#         # Paginate inquiries
#         inquiry_page = request.GET.get('inquiry_page', 1)
#         inquiry_paginator = Paginator(inquiries, 10)  # Show 10 inquiries per page
#         try:
#             inquiries = inquiry_paginator.page(inquiry_page)
#         except PageNotAnInteger:
#             inquiries = inquiry_paginator.page(1)
#         except EmptyPage:
#             inquiries = inquiry_paginator.page(inquiry_paginator.num_pages)

#         # Render the data
#         context = {
#             "product": product,
#             "product_id": product.product_id,
#             "product_name": product.product_name,
#             "stock_adjustments": stock_adjustments,
#             "inquiries": inquiries,
#             "total_amount_from_inquiries": total_amount_from_inquiries,
#             "monthly_sales": sales_data
#         }
        
#         return render(request, "inventory/product/product_detail.html", context)
#     except Product.DoesNotExist:
#         return Response(status=status.HTTP_404_NOT_FOUND, data={'error': 'Product not found'})
from rest_framework.response import Response
from rest_framework import status
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.db.models import Sum, Count
from django.db.models.functions import ExtractMonth
import datetime

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status
from .models import Product

from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from .models import Product

@api_view(['GET'])
def get_product_details_api(request, product_id):
    try:
        product = Product.objects.get(pk=product_id)

        # Prepare the data to be sent as JSON
        product_data = {
            "product_id": product.product_id,
            "product_name": product.product_name,
            "price": product.price,
            "discounted_price": product.discounted_price,
            "quantity_in_stock": product.quantity_in_stock,
            "uom": product.uom.unit if product.uom else None,
            "UOMSTR": product.uom.unit if product.uom else None,
            "productType": product.productType,
            "sku": product.sku,
            "barcode": product.barcode,
            "weight": product.weight,
            "dimensions": product.dimensions,
            "gauge_name": product.gauge.name if product.gauge else None,
            "gauge_id": product.gauge.gauge_id if product.gauge else None,
        }
        
        return Response(product_data, status=status.HTTP_200_OK)
    
    except Product.DoesNotExist:
        return Response({"error": "Product not found"}, status=status.HTTP_404_NOT_FOUND)


from django.db.models import Sum
from django.db.models import Count
from django.db.models.functions import ExtractMonth



from django.db.models import Count
import datetime

@api_view(['GET'])
def get_product_details(request, product_id):
    try:
        product = Product.objects.get(pk=product_id)
        # Get stock adjustments for the product
        stock_adjustments = Stock.objects.filter(product=product)

        # Get inquiries related to the product
        inquiries = Inquiry.objects.filter(product=product)

        # Calculate total amount made from inquiries
        total_amount_from_inquiries = inquiries.aggregate(total_amount=Sum('amount'))['total_amount']

        # Query sales data for the current year
        current_year = datetime.datetime.now().year
        monthly_sales = inquiries.filter(created_at__year=current_year).annotate(
            month=ExtractMonth('created_at')).values('month').annotate(
            total_sales=Count('id')).order_by('month')

        # Prepare data for monthly sales chart
        sales_data = {
            'labels': [datetime.date(2000, month, 1).strftime('%B') for month in range(1, 13)],  # Month names
            'data': [monthly_sales.get(month=month)['total_sales'] if monthly_sales.filter(month=month).exists() else 0 for month in range(1, 13)]  # Sales counts
        }

        # Render the data or serialize it as needed
        product_data = {
            "product": product,
            "product_id": product.product_id,
            "product_name": product.product_name,
            "stock_adjustments": list(stock_adjustments.values()),  # Convert QuerySet to list for JSON response
            "inquiries": list(inquiries.values()),  # Convert QuerySet to list for JSON response
            "total_amount_from_inquiries": total_amount_from_inquiries,
            "monthly_sales": sales_data
        }
        
        return render(request,"inventory/product/product_detail.html",product_data)
    except Product.DoesNotExist:
        return Response(status=status.HTTP_404_NOT_FOUND, data={'error': 'Product not found'})




